/***
    Copyright (C) 2013 Cable Developers

    This program or library is free software; you can redistribute it
    and/or modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 3 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General
    Public License along with this library; if not, write to the
    Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301 USA.
***/

public class Cable.Controller.Channel : GLib.Object {

    public weak Server server;
    public Port.View.Channel view;
    public Port.Irc.Channel backend;

    public string name; //TODO id

    public Channel (Server server, string name) {
        this.server = server;
        this.name = name;

        initialize ();
        connect_signals ();
        load_state ();
    }
    
    void initialize () {
        if (server.view.has_chat (name) == false)
            server.view.add_channel (name, name);

        view = server.view.get_chat (name) as Port.View.Channel;
        view.nick = server.nick;
        backend = server.backend.get_or_create_channel (name);
    }

    void connect_signals () {
        view.part.connect (part_channel);
        view.join.connect (start_joining);
        view.private_chat_requested.connect (open_new_private_chat_and_select);
        view.send_message.connect (send_message_or_command);
        view.focused.connect (clear_launcher_alerts);
        
        server.backend.event_nick.connect (on_nick_changed);
        backend.event_message.connect (increase_unread_messages);
        backend.event_topic.connect (change_topic);
        backend.event_part.connect (on_user_parted);
        backend.event_join.connect (on_joined);
        backend.event_action.connect (display_action);
        backend.event_names.connect (view.list_users);
        backend.event_no_such.connect (display_no_such);
        backend.event_message.connect (display_message);
    }
    
    void load_state () {
        view.state = State.JOINING;

        if (server.backend.connected == false) {
            server.backend.event_connected.connect (() => {
                if (backend.joined)
                    backend.send_part ();
                backend.send_join ();
            });
        } else {
            if (backend.joined)
                backend.send_part ();
            backend.send_join ();
        }
    }
    
    void on_joined (string from) {
        if (server.backend.is_me (from))
            display_joined ();
        else
            add_user (from);
    }
    
    void display_joined () {
        view.state = State.CHAT;
    }
    
    void add_user (string from) {
        var name = from.split ("!")[0];
        var type = remove_prefix_and_get_type (ref name);
        view.add_user (name, type);
    }
    
    void display_message (string from, string message) {
        var nick = from.split ("!") [0];

        if (message.contains (server.nick)) {

            if (view.has_focus == false) {
                Services.Notify.send (nick, message);
                Services.Launcher.jobs++;
                Services.Launcher.urgent = true;
            }

            view.message (nick, message, MessageType.PING);
        } else {
            view.message (nick, message);
        }
    }
    
    void display_no_such () {
        view.state = State.NO_SUCH;
    }
    
    void display_action (string from, string message) {
        var nick = from.split ("!")[0];
        view.message ("", @"*$nick $message*", MessageType.ACTION);
    }

    void part_channel () {
        backend.send_part ();
    }

    void start_joining () {
        view.state = State.JOINING;
        backend.send_join ();
    }

    void open_new_private_chat_and_select (string name) {
        server.open_new_chat_with (name);
        if (server.private_chats.has_key (name))
            server.window.view.current_chat = server.private_chats [name].view;
    }

    void send_message_or_command (string message) {
        if (message.has_prefix ("/"))
            send_command (message.replace ("/", ""));
        else
            server.backend.send_message (name, message);
        view.entry = "";
    }

    bool clear_launcher_alerts () {
        Services.Launcher.jobs = 0;
        Services.Launcher.urgent = false;
        return false;
    }

    void increase_unread_messages () {
        if (is_not_current_chat ())
            view.unread_messages++;
    }

    bool is_not_current_chat () {
        return server.window.view.current_chat != view;
    }

    void change_topic (string from, string new_topic) {
        view.topic = new_topic;
    }

    void on_nick_changed (string from, string new_nick) {
        if (server.backend.is_me (from))
            change_my_nick (new_nick);
        change_other_nick (from, new_nick);
    }

    void change_my_nick (string new_nick) {
        view.nick = new_nick;
    }

    void change_other_nick (string from, string new_nick) {
        var name = from.split ("!")[0];

        if (!view.has_user (name))
            return;

        var type = remove_prefix_and_get_type (ref name);
        view.rename_user (name, new_nick, type);
    }

    void on_user_parted (string from, string message) {
        if (server.backend.is_me (from))
            display_not_joined ();
        else
            remove_user (from, message);
    }

    void display_not_joined () {
        view.state = State.NOT_JOINED;
    }

    void remove_user (string from, string message) {
        var name = from.split ("!")[0];
        remove_prefix_and_get_type (ref name);
        view.remove_user (name, message);
    }

    UserType remove_prefix_and_get_type (ref string nick) {
        if (name.has_prefix ("@")) {
            name.replace ("@", "");
            return UserType.OPERATOR;
        }

        if (name.has_prefix ("+")) {
            name.replace ("+", "");
            return UserType.VOICED;
        }

        return UserType.REGULAR;
    }

    void send_command (string command) {

        var args = command.split (" ");

        switch (args[0]) {
            case "join":
                return_if_fail (args.length > 1);
                var channel_with_hash = args[1];
                if (channel_with_hash.slice(0, 1) != "#") {
                    channel_with_hash = "#" + args[1];
                }
                var channel = new Channel (server, channel_with_hash);
                server.channels[channel_with_hash] = channel;
                server.window.view.current_chat = channel.view;
                break;

            case "part":
                backend.send_part ();
                break;

            case "quit":
                server.backend.send_quit ();
                break;

            case "nick":
                return_if_fail (args.length > 1);
                server.backend.send_nick (args[1]);
                break;

            case "topic":
                return_if_fail (args.length > 1);
                backend.send_topic (string.joinv (" ", args[1:-1]));
                break;

            case "me":
            case "action":
                return_if_fail (args.length > 1);
                backend.send_action (string.joinv (" ", args[1:-1]));
                break;

            case "kick":
                return_if_fail (args.length > 2);
                backend.send_kick (args[1], string.joinv (" ", args[2:-1]));
                break;

            case "ignore":
                return_if_fail (args.length > 1);
                server.backend.send_ignore (string.joinv (" ", args[1:-1]));
                break;

            case "unignore":
                return_if_fail (args.length > 1);
                server.backend.send_unignore (string.joinv (" ", args[1:-1]));
                break;

            case "msg":
                return_if_fail (args.length > 2);
                server.backend.send_message (args[1], string.joinv (" ", args[2:-1]));
                break;

            case "notice":
                return_if_fail (args.length > 2);
                server.backend.send_notice (args[1], string.joinv (" ", args[2:-1]));
                break;

            case "away":
                if (args.length < 2)
                    server.backend.send_back ();
                else
                    server.backend.send_away (string.joinv (" ", args[1:-1]));
                break;

            default:
                server.backend.send_raw (command);
                break;
        }
    }
}
