/***
    Copyright (C) 2013 Cable Developers

    This program or library is free software; you can redistribute it
    and/or modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 3 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General
    Public License along with this library; if not, write to the
    Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301 USA.
***/

public class Cable.Utils.Identity : GLib.Object {

    public string nick_name { get; construct; }
    public string real_name { get; set; }
    public string user_name { get; set; }
    public string part_message { get; set; }
    public string away_message { get; set; }
    public string quit_message { get; set; }

    static Gee.ArrayList <Identity> _all;
    public static Gee.List <Identity> all {
        owned get {
            return _all.read_only_view as Gee.List;
        }
    }

    private Identity (string nick_name,
                      string? real_name = null,
                      string? user_name = null,
                      string? part_message = null,
                      string? quit_message = null,
                      string? away_message = null)
        requires (Identity.search (nick_name) == null)
        ensures (search (nick_name) == this) {

        GLib.Object (nick_name: nick_name,
                     real_name: real_name ?? nick_name,
                     part_message: part_message ?? _("Good night everyone!"),
                     quit_message: quit_message ?? _("Good night everyone!"),
                     away_message: away_message ?? _("Try again later!"));

        _all.add (this);

        notify.connect (() => {
            Settings.preferences.identities = export ();
        });
    }

    ~Identity () {
        _all.remove (this);
    }

    public static void init ()
        requires (_all == null) {

        _all = new Gee.ArrayList <Identity> ();
        import (Settings.preferences.identities);

        if (Settings.preferences.identities.length == 0) {
            get_default ();
            Settings.preferences.identities = export ();
        }
    }

    static void import (string[] data) {
        foreach (var identity in data) {
            var values = identity.split (":");

            switch (values.length) {
                case 0:
                    warn_if_reached ();
                    break;
                case 1:
                    new Identity (values[0]);
                    break;
                case 2:
                    new Identity (values[0], values[1]);
                    break;
                case 3:
                    new Identity (values[0], values[1], values[2]);
                    break;
                case 4:
                    new Identity (values[0], values[1], values[2], values[3]);
                    break;
                case 5:
                    new Identity (values[0], values[1], values[2], values[3],
                                  values[4]);
                    break;
                case 6:
                    new Identity (values[0], values[1], values[2], values[3],
                                  values[4], values[5]);
                    break;
                default:
                    warn_if_reached ();
                    break;
            }
        }
    }

    static string[] export () {
        string[] result = {};

        foreach (var identity in all) {
            result += string.join (":",
                identity.nick_name,
                identity.real_name,
                identity.part_message,
                identity.quit_message,
                identity.away_message);
        }

        return result;
    }

    public static void remove (string nick)
        requires (search (nick) != null)
        ensures (search (nick) == null) {

        _all.remove (search (nick));
        Settings.preferences.identities = export ();
    }

    public static Identity get_or_create (string nick)
        ensures (search (nick) != null) {
        var id = search (nick) ?? new Identity (nick);
        Settings.preferences.identities = export ();
        return id;
    }

    static Identity? search (string nick_name) {
        foreach (var identity in _all)
            if (identity.nick_name == nick_name)
                return identity;

        return (Identity) null;
    }

    /**
     * Creates the default identity from your current login.
     * @returns Default identity.
     */
    static Identity get_default ()
        requires (search (GLib.Environment.get_user_name ()) == null) {

        return new Identity (GLib.Environment.get_user_name (),
                             GLib.Environment.get_real_name ());
    }
}
